@@ -192,15 +192,16 @@ SlowBuffer.prototype.slice = function(start, end) {
 
 function coerce(length) {
   // Coerce length to a number (possibly NaN), round up
-  // in case it's fractional (e.g. 123.456) then do a
-  // double negate to coerce a NaN to 0. Easy, right?
-  length = ~~Math.ceil(+length);
-  return length < 0 ? 0 : length;
+  // in case it's fractional (e.g. 123.456). Since NaN
+  // comparisons are always false, use to return zero.
+  length = Math.ceil(+length);
+  return length > 0 ? length : 0;
 }
 
 
-// Buffer
+var zeroBuffer = new SlowBuffer(0);
 
+// Buffer
 function Buffer(subject, encoding, offset) {
   if (!(this instanceof Buffer)) {
     return new Buffer(subject, encoding, offset);
@@ -242,13 +243,18 @@ function Buffer(subject, encoding, offset) {
       this.parent = new SlowBuffer(this.length);
       this.offset = 0;
 
-    } else {
+    } else if (this.length > 0) {
       // Small buffer.
       if (!pool || pool.length - pool.used < this.length) allocPool();
       this.parent = pool;
       this.offset = pool.used;
       pool.used += this.length;
       if (pool.used & 7) pool.used = (pool.used + 8) & ~7;
+
+    } else {
+      // Zero-length buffer
+      this.parent = zeroBuffer;
+      this.offset = 0;
     }
 
     // Treat array-ish objects as a byte array.
@@ -551,7 +557,7 @@ Buffer.prototype.slice = function(start, end) {
   if (end === undefined) end = this.length;
   if (end > this.length) throw new Error('oob');
   if (start > end) throw new Error('oob');
-
+  if (start < 0) throw new Error('start out of bounds');
   return new Buffer(this.parent, end - start, +start + this.offset);
 };
 
