@@ -300,6 +300,9 @@ function setupChannel(target, channel) {
     } else {
       this.buffering = false;
       target.disconnect();
+      channel.onread = nop;
+      channel.close();
+      maybeClose(target);
     }
   };
 
@@ -313,7 +316,8 @@ function setupChannel(target, channel) {
     var obj = handleConversion[message.type];
 
     // Update simultaneous accepts on Windows
-    if (obj.simultaneousAccepts) {
+    if (process.platform === 'win32') {
+      handle._simultaneousAccepts = false;
       net._setSimultaneousAccepts(handle);
     }
 
@@ -709,9 +713,10 @@ ChildProcess.prototype.spawn = function(options) {
   }
 
   // At least 3 stdio will be created
-  if (stdio.length < 3) {
-    stdio = stdio.concat(new Array(3 - stdio.length));
-  }
+  // Don't concat() a new Array() because it would be sparse, and
+  // stdio.reduce() would skip the sparse elements of stdio.
+  // See http://stackoverflow.com/a/5501711/3561
+  while (stdio.length < 3) stdio.push(undefined);
 
   // Translate stdio into C++-readable form
   // (i.e. PipeWraps or fds)
@@ -743,7 +748,7 @@ ChildProcess.prototype.spawn = function(options) {
       ipc = createPipe(true);
       ipcFd = i;
 
-      acc.push({ type: 'pipe', handle: ipc });
+      acc.push({ type: 'pipe', handle: ipc, ipc: true });
     } else if (typeof stdio === 'number' || typeof stdio.fd === 'number') {
       acc.push({ type: 'fd', fd: stdio.fd || stdio });
     } else if (getHandleWrapType(stdio) || getHandleWrapType(stdio.handle) ||
@@ -794,6 +799,11 @@ ChildProcess.prototype.spawn = function(options) {
   stdio.forEach(function(stdio, i) {
     if (stdio.type === 'ignore') return;
 
+    if (stdio.ipc) {
+      self._closesNeeded++;
+      return;
+    }
+
     if (stdio.handle) {
       // when i === 0 - we're dealing with stdin
       // (which is the only one writable pipe)
