@@ -401,6 +401,7 @@ IncomingMessage.prototype._addHeaderLine = function(field, value) {
     case 'pragma':
     case 'link':
     case 'www-authenticate':
+    case 'proxy-authenticate':
     case 'sec-websocket-extensions':
     case 'sec-websocket-protocol':
       if (field in dest) {
@@ -758,6 +759,10 @@ OutgoingMessage.prototype.addTrailers = function(headers) {
 };
 
 
+var zero_chunk_buf = new Buffer('\r\n0\r\n');
+var crlf_buf = new Buffer('\r\n');
+
+
 OutgoingMessage.prototype.end = function(data, encoding) {
   if (this.finished) {
     return false;
@@ -775,8 +780,7 @@ OutgoingMessage.prototype.end = function(data, encoding) {
   var ret;
 
   var hot = this._headerSent === false &&
-            typeof(data) === 'string' &&
-            data.length > 0 &&
+            (data && data.length > 0) &&
             this.output.length === 0 &&
             this.connection &&
             this.connection.writable &&
@@ -788,13 +792,73 @@ OutgoingMessage.prototype.end = function(data, encoding) {
     //   res.end(blah);
     // HACKY.
 
-    if (this.chunkedEncoding) {
-      var l = Buffer.byteLength(data, encoding).toString(16);
-      ret = this.connection.write(this._header + l + CRLF +
-                                  data + '\r\n0\r\n' +
-                                  this._trailer + '\r\n', encoding);
+    if (typeof data === 'string') {
+      if (this.chunkedEncoding) {
+        var l = Buffer.byteLength(data, encoding).toString(16);
+        ret = this.connection.write(this._header + l + CRLF +
+                                    data + '\r\n0\r\n' +
+                                    this._trailer + '\r\n', encoding);
+      } else {
+        ret = this.connection.write(this._header + data, encoding);
+      }
+    } else if (Buffer.isBuffer(data)) {
+      if (this.chunkedEncoding) {
+        var chunk_size = data.length.toString(16);
+
+        // Skip expensive Buffer.byteLength() calls; only ISO-8859-1 characters
+        // are allowed in HTTP headers. Therefore:
+        //
+        //   this._header.length == Buffer.byteLength(this._header.length)
+        //   this._trailer.length == Buffer.byteLength(this._trailer.length)
+        //
+        var header_len = this._header.length;
+        var chunk_size_len = chunk_size.length;
+        var data_len = data.length;
+        var trailer_len = this._trailer.length;
+
+        var len = header_len +
+                  chunk_size_len +
+                  2 + // '\r\n'.length
+                  data_len +
+                  5 + // '\r\n0\r\n'.length
+                  trailer_len +
+                  2;  // '\r\n'.length
+
+        var buf = new Buffer(len);
+        var off = 0;
+
+        buf.write(this._header, off, header_len, 'ascii');
+        off += header_len;
+
+        buf.write(chunk_size, off, chunk_size_len, 'ascii');
+        off += chunk_size_len;
+
+        crlf_buf.copy(buf, off);
+        off += 2;
+
+        data.copy(buf, off);
+        off += data_len;
+
+        zero_chunk_buf.copy(buf, off);
+        off += 5;
+
+        if (trailer_len > 0) {
+          buf.write(this._trailer, off, trailer_len, 'ascii');
+          off += trailer_len;
+        }
+
+        crlf_buf.copy(buf, off);
+
+        ret = this.connection.write(buf);
+      } else {
+        var header_len = this._header.length;
+        var buf = new Buffer(header_len + data.length);
+        buf.write(this._header, 0, header_len, 'ascii');
+        data.copy(buf, header_len);
+        ret = this.connection.write(buf);
+      }
     } else {
-      ret = this.connection.write(this._header + data, encoding);
+      throw new TypeError('first argument must be a string or Buffer');
     }
     this._headerSent = true;
 
@@ -891,7 +955,7 @@ function ServerResponse(req) {
   this.sendDate = true;
 
   if (req.httpVersionMajor < 1 || req.httpVersionMinor < 1) {
-    this.useChunkedEncodingByDefault = false;
+    this.useChunkedEncodingByDefault = chunkExpression.test(req.headers.te);
     this.shouldKeepAlive = false;
   }
 }
@@ -903,7 +967,24 @@ exports.ServerResponse = ServerResponse;
 ServerResponse.prototype.statusCode = 200;
 
 function onServerResponseClose() {
-  this._httpMessage.emit('close');
+  // EventEmitter.emit makes a copy of the 'close' listeners array before
+  // calling the listeners. detachSocket() unregisters onServerResponseClose
+  // but if detachSocket() is called, directly or indirectly, by a 'close'
+  // listener, onServerResponseClose is still in that copy of the listeners
+  // array. That is, in the example below, b still gets called even though
+  // it's been removed by a:
+  //
+  //   var obj = new events.EventEmitter;
+  //   obj.on('event', a);
+  //   obj.on('event', b);
+  //   function a() { obj.removeListener('event', b) }
+  //   function b() { throw "BAM!" }
+  //   obj.emit('event');  // throws
+  //
+  // Ergo, we need to deal with stale 'close' events and handle the case
+  // where the ServerResponse object has already been deconstructed.
+  // Fortunately, that requires only a single if check. :-)
+  if (this._httpMessage) this._httpMessage.emit('close');
 }
 
 ServerResponse.prototype.assignSocket = function(socket) {
@@ -1067,7 +1148,7 @@ Agent.prototype.addRequest = function(req, host, port, localAddress) {
   }
   if (this.sockets[name].length < this.maxSockets) {
     // If we are under maxSockets create a new one.
-    req.onSocket(this.createSocket(name, host, port, localAddress));
+    req.onSocket(this.createSocket(name, host, port, localAddress, req));
   } else {
     // We are over limit so we'll add it to the queue.
     if (!this.requests[name]) {
@@ -1076,12 +1157,21 @@ Agent.prototype.addRequest = function(req, host, port, localAddress) {
     this.requests[name].push(req);
   }
 };
-Agent.prototype.createSocket = function(name, host, port, localAddress) {
+Agent.prototype.createSocket = function(name, host, port, localAddress, req) {
   var self = this;
   var options = util._extend({}, self.options);
   options.port = port;
   options.host = host;
   options.localAddress = localAddress;
+
+  options.servername = host;
+  if (req) {
+    var hostHeader = req.getHeader('host');
+    if (hostHeader) {
+      options.servername = hostHeader.replace(/:.*$/, '');
+    }
+  }
+
   var s = self.createConnection(options);
   if (!self.sockets[name]) {
     self.sockets[name] = [];
@@ -1122,8 +1212,9 @@ Agent.prototype.removeSocket = function(s, name, host, port, localAddress) {
     }
   }
   if (this.requests[name] && this.requests[name].length) {
+    var req = this.requests[name][0];
     // If we have pending requests and a socket gets closed a new one
-    this.createSocket(name, host, port, localAddress).emit('free');
+    this.createSocket(name, host, port, localAddress, req).emit('free');
   }
 };
 
@@ -1357,7 +1448,9 @@ function socketOnData(d, start, end) {
   if (ret instanceof Error) {
     debug('parse error');
     freeParser(parser, req);
-    socket.destroy(ret);
+    socket.destroy();
+    req.emit('error', ret);
+    req._hadError = true;
   } else if (parser.incoming && parser.incoming.upgrade) {
     // Upgrade or CONNECT
     var bytesParsed = ret;
@@ -1402,6 +1495,12 @@ function parserOnIncomingClient(res, shouldKeepAlive) {
   var socket = this.socket;
   var req = socket._httpMessage;
 
+  // propogate "domain" setting...
+  if (req.domain && !res.domain) {
+    debug('setting "res.domain"');
+    res.domain = req.domain;
+  }
+
   debug('AGENT incoming response!');
 
   if (req.res) {
@@ -1464,6 +1563,10 @@ function responseOnEnd() {
     assert(!socket.writable);
   } else {
     debug('AGENT socket keep-alive');
+    if (req.timeoutCb) {
+      socket.setTimeout(0, req.timeoutCb);
+      req.timeoutCb = null;
+    }
     socket.removeListener('close', socketCloseListener);
     socket.removeListener('error', socketErrorListener);
     socket.emit('free');
@@ -1482,9 +1585,7 @@ ClientRequest.prototype.onSocket = function(socket) {
     parser.incoming = null;
     req.parser = parser;
 
-    parser.socket = socket;
     socket.parser = parser;
-    parser.incoming = null;
     socket._httpMessage = req;
 
     // Setup "drain" propogation.
@@ -1546,6 +1647,9 @@ ClientRequest.prototype.setTimeout = function(msecs, callback) {
   }
 
   if (this.socket && this.socket.writable) {
+    if (this.timeoutCb)
+      this.socket.setTimeout(0, this.timeoutCb);
+    this.timeoutCb = emitTimeout;
     this.socket.setTimeout(msecs, emitTimeout);
     return;
   }
