@@ -24,6 +24,7 @@ var Stream = require('stream');
 var timers = require('timers');
 var util = require('util');
 var assert = require('assert');
+var cares = process.binding('cares_wrap');
 var cluster;
 
 function noop() {}
@@ -106,6 +107,7 @@ function normalizeConnectArgs(args) {
   var cb = args[args.length - 1];
   return (typeof cb === 'function') ? [options, cb] : [options];
 }
+exports._normalizeConnectArgs = normalizeConnectArgs;
 
 
 /* called when creating new Socket, or when re-using a closed Socket */
@@ -131,19 +133,25 @@ function Socket(options) {
 
   Stream.call(this);
 
-  if (typeof options == 'number') {
-    // Legacy interface.
-    var fd = options;
-    this._handle = createPipe();
-    this._handle.open(fd);
-    this.readable = this.writable = true;
-    initSocketHandle(this);
-  } else {
-    // private
-    this._handle = options && options.handle;
-    initSocketHandle(this);
-    this.allowHalfOpen = options && options.allowHalfOpen;
+  switch (typeof options) {
+    case 'number':
+      options = { fd: options }; // Legacy interface.
+      break;
+    case 'undefined':
+      options = {};
+      break;
   }
+
+  if (typeof options.fd === 'undefined') {
+    this._handle = options && options.handle; // private
+  } else {
+    this._handle = createPipe();
+    this._handle.open(options.fd);
+    this.readable = this.writable = true;
+  }
+
+  initSocketHandle(this);
+  this.allowHalfOpen = options && options.allowHalfOpen;
 }
 util.inherits(Socket, Stream);
 
@@ -160,7 +168,7 @@ Socket.prototype.listen = function() {
 
 
 Socket.prototype.setTimeout = function(msecs, callback) {
-  if (msecs > 0) {
+  if (msecs > 0 && !isNaN(msecs) && isFinite(msecs)) {
     timers.enroll(this, msecs);
     timers.active(this);
     if (callback) {
@@ -230,11 +238,7 @@ Object.defineProperty(Socket.prototype, 'bufferSize', {
 
 Socket.prototype.pause = function() {
   this._paused = true;
-  if (this._connecting) {
-    // will actually pause once the handle is established.
-    return;
-  }
-  if (this._handle) {
+  if (this._handle && !this._connecting) {
     this._handle.readStop();
   }
 };
@@ -242,11 +246,7 @@ Socket.prototype.pause = function() {
 
 Socket.prototype.resume = function() {
   this._paused = false;
-  if (this._connecting) {
-    // will actually resume once the handle is established.
-    return;
-  }
-  if (this._handle) {
+  if (this._handle && !this._connecting) {
     this._handle.readStart();
   }
 };
@@ -377,7 +377,7 @@ Socket.prototype.destroy = function(exception) {
 function onread(buffer, offset, length) {
   var handle = this;
   var self = handle.owner;
-  assert.equal(handle, self._handle);
+  assert(handle === self._handle, 'handle != self._handle');
 
   timers.active(self);
 
@@ -666,7 +666,7 @@ Socket.prototype.connect = function(options, cb) {
   }
 
   if (typeof cb === 'function') {
-    self.on('connect', cb);
+    self.once('connect', cb);
   }
 
   timers.active(this);
@@ -724,25 +724,19 @@ function afterConnect(status, handle, req, readable, writable) {
     return;
   }
 
-  assert.equal(handle, self._handle);
+  assert(handle === self._handle, 'handle != self._handle');
 
   debug('afterConnect');
 
   assert.ok(self._connecting);
   self._connecting = false;
 
-  // now that we're connected, process any pending pause state.
-  if (self._paused) {
-    self._paused = false;
-    self.pause();
-  }
-
   if (status == 0) {
     self.readable = readable;
     self.writable = writable;
     timers.active(self);
 
-    if (self.readable) {
+    if (self.readable && !self._paused) {
       handle.readStart();
     }
 
@@ -851,6 +845,7 @@ var createServerHandle = exports._createServerHandle =
       default:
         // Not a fd we can listen on.  This will trigger an error.
         debug('listen invalid fd=' + fd + ' type=' + type);
+        global.errno = 'EINVAL'; // hack, callers expect that errno is set
         handle = null;
         break;
     }
@@ -895,8 +890,9 @@ Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
   if (!self._handle) {
     self._handle = createServerHandle(address, port, addressType, fd);
     if (!self._handle) {
+      var error = errnoException(errno, 'listen');
       process.nextTick(function() {
-        self.emit('error', errnoException(errno, 'listen'));
+        self.emit('error', error);
       });
       return;
     }
@@ -911,10 +907,11 @@ Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
   r = self._handle.listen(backlog || 511);
 
   if (r) {
+    var ex = errnoException(errno, 'listen');
     self._handle.close();
     self._handle = null;
     process.nextTick(function() {
-      self.emit('error', errnoException(errno, 'listen'));
+      self.emit('error', ex);
     });
     return;
   }
@@ -959,9 +956,8 @@ Server.prototype.listen = function() {
   var TCP = process.binding('tcp_wrap').TCP;
 
   if (arguments.length == 0 || typeof arguments[0] == 'function') {
-    // Don't bind(). OS will assign a port with INADDR_ANY.
-    // The port can be found with server.address()
-    listen(self, null, null, backlog);
+    // Bind to a random port.
+    listen(self, '0.0.0.0', 0, null, backlog);
 
   } else if (arguments[0] && typeof arguments[0] === 'object') {
     var h = arguments[0];
@@ -973,7 +969,7 @@ Server.prototype.listen = function() {
     if (h instanceof TCP) {
       self._handle = h;
       listen(self, null, -1, -1, backlog);
-    } else if (h.fd && typeof h.fd === 'number' && h.fd >= 0) {
+    } else if (typeof h.fd === 'number' && h.fd >= 0) {
       listen(self, null, null, null, backlog, h.fd);
     } else {
       throw new Error('Invalid listen argument: ' + h);
@@ -1034,7 +1030,7 @@ function onconnection(clientHandle) {
   });
   socket.readable = socket.writable = true;
 
-  socket.resume();
+  clientHandle.readStart();
 
   self._connections++;
   socket.server = self;
@@ -1096,27 +1092,7 @@ Server.prototype._setupSlave = function(socketList) {
 
 // TODO: isIP should be moved to the DNS code. Putting it here now because
 // this is what the legacy system did.
-// NOTE: This does not accept IPv6 with an IPv4 dotted address at the end,
-//  and it does not detect more than one double : in a string.
-exports.isIP = function(input) {
-  if (!input) {
-    return 0;
-  } else if (/^(\d?\d?\d)\.(\d?\d?\d)\.(\d?\d?\d)\.(\d?\d?\d)$/.test(input)) {
-    var parts = input.split('.');
-    for (var i = 0; i < parts.length; i++) {
-      var part = parseInt(parts[i]);
-      if (part < 0 || 255 < part) {
-        return 0;
-      }
-    }
-    return 4;
-  } else if (/^::|^::1|^([a-fA-F0-9]{1,4}::?){1,7}([a-fA-F0-9]{1,4})$/.test(
-      input)) {
-    return 6;
-  } else {
-    return 0;
-  }
-};
+exports.isIP = cares.isIP;
 
 
 exports.isIPv4 = function(input) {
