@@ -22,12 +22,16 @@
 var crypto = require('crypto');
 var util = require('util');
 var net = require('net');
+var url = require('url');
 var events = require('events');
 var Stream = require('stream');
 var END_OF_FILE = 42;
 var assert = require('assert').ok;
 var constants = require('constants');
 
+var DEFAULT_CIPHERS = 'ECDHE-RSA-AES128-SHA256:AES128-GCM-SHA256:' + // TLS 1.2
+                      'RC4:HIGH:!MD5:!aNULL:!EDH';                   // TLS 1.0
+
 // Allow {CLIENT_RENEG_LIMIT} client-initiated session renegotiations
 // every {CLIENT_RENEG_WINDOW} seconds. An error event is emitted if more
 // renegotations are seen. The settings are applied to all remote client
@@ -77,6 +81,155 @@ function convertNPNProtocols(NPNProtocols, out) {
   }
 }
 
+
+function checkServerIdentity(host, cert) {
+  // Create regexp to much hostnames
+  function regexpify(host, wildcards) {
+    // Add trailing dot (make hostnames uniform)
+    if (!/\.$/.test(host)) host += '.';
+
+    // The same applies to hostname with more than one wildcard,
+    // if hostname has wildcard when wildcards are not allowed,
+    // or if there are less than two dots after wildcard (i.e. *.com or *d.com)
+    //
+    // also
+    //
+    // "The client SHOULD NOT attempt to match a presented identifier in
+    // which the wildcard character comprises a label other than the
+    // left-most label (e.g., do not match bar.*.example.net)."
+    // RFC6125
+    if (!wildcards && /\*/.test(host) || /[\.\*].*\*/.test(host) ||
+        /\*/.test(host) && !/\*.*\..+\..+/.test(host)) {
+      return /$./;
+    }
+
+    // Replace wildcard chars with regexp's wildcard and
+    // escape all characters that have special meaning in regexps
+    // (i.e. '.', '[', '{', '*', and others)
+    var re = host.replace(
+        /\*([a-z0-9\\-_\.])|[\.,\-\\\^\$+?*\[\]\(\):!\|{}]/g,
+        function(all, sub) {
+          if (sub) return '[a-z0-9\\-_]*' + (sub === '-' ? '\\-' : sub);
+          return '\\' + all;
+        });
+
+    return new RegExp('^' + re + '$', 'i');
+  }
+
+  var dnsNames = [],
+      uriNames = [],
+      ips = [],
+      matchCN = true,
+      valid = false;
+
+  // There're several names to perform check against:
+  // CN and altnames in certificate extension
+  // (DNS names, IP addresses, and URIs)
+  //
+  // Walk through altnames and generate lists of those names
+  if (cert.subjectaltname) {
+    matchCN = false;
+    cert.subjectaltname.split(/, /g).forEach(function(altname) {
+      if (/^DNS:/.test(altname)) {
+        dnsNames.push(altname.slice(4));
+      } else if (/^IP Address:/.test(altname)) {
+        ips.push(altname.slice(11));
+      } else if (/^URI:/.test(altname)) {
+        var uri = url.parse(altname.slice(4));
+        if (uri) uriNames.push(uri.hostname);
+      }
+    });
+  }
+
+  // If hostname is an IP address, it should be present in the list of IP
+  // addresses.
+  if (net.isIP(host)) {
+    valid = ips.some(function(ip) {
+      return ip === host;
+    });
+  } else {
+    // Transform hostname to canonical form
+    if (!/\.$/.test(host)) host += '.';
+
+    // Otherwise check all DNS/URI records from certificate
+    // (with allowed wildcards)
+    dnsNames = dnsNames.map(function(name) {
+      return regexpify(name, true);
+    });
+
+    // Wildcards ain't allowed in URI names
+    uriNames = uriNames.map(function(name) {
+      return regexpify(name, false);
+    });
+
+    dnsNames = dnsNames.concat(uriNames);
+
+    if (dnsNames.length > 0) matchCN = false;
+
+    // Match against Common Name (CN) only if altnames are not present.
+    //
+    // "As noted, a client MUST NOT seek a match for a reference identifier
+    //  of CN-ID if the presented identifiers include a DNS-ID, SRV-ID,
+    //  URI-ID, or any application-specific identifier types supported by the
+    //  client."
+    // RFC6125
+    if (matchCN) {
+      var commonNames = cert.subject.CN;
+      if (Array.isArray(commonNames)) {
+        for (var i = 0, k = commonNames.length; i < k; ++i) {
+          dnsNames.push(regexpify(commonNames[i], true));
+        }
+      } else {
+        dnsNames.push(regexpify(commonNames, true));
+      }
+    }
+
+    valid = dnsNames.some(function(re) {
+      return re.test(host);
+    });
+  }
+
+  return valid;
+}
+exports.checkServerIdentity = checkServerIdentity;
+
+
+
+function SlabBuffer() {
+  this.create();
+}
+
+
+SlabBuffer.prototype.create = function create() {
+  this.isFull = false;
+  this.pool = new Buffer(10 * 1024 * 1024);
+  this.offset = 0;
+  this.remaining = this.pool.length;
+};
+
+
+SlabBuffer.prototype.use = function use(context, fn) {
+  if (this.remaining === 0) {
+    this.isFull = true;
+    return 0;
+  }
+
+  var bytes = fn.call(context, this.pool, this.offset, this.remaining);
+
+  if (bytes > 0) {
+    this.offset += bytes;
+    this.remaining -= bytes;
+  }
+
+  assert(this.remaining >= 0);
+
+  return bytes;
+};
+
+
+var slabBuffer = new SlabBuffer();
+
+
 // Base class of both CleartextStream and EncryptedStream
 function CryptoStream(pair) {
   Stream.call(this);
@@ -90,6 +243,7 @@ function CryptoStream(pair) {
   this._pending = [];
   this._pendingCallbacks = [];
   this._pendingBytes = 0;
+  this._buffer = slabBuffer;
 }
 util.inherits(CryptoStream, Stream);
 
@@ -339,18 +493,13 @@ CryptoStream.prototype._push = function() {
   }
 
   while (!this._paused) {
-    var chunkBytes = 0;
-    if (!this._pool || (this._poolStart >= this._poolEnd)) {
-      this._pool = new Buffer(16 * 4096);
-      this._poolStart = 0;
-      this._poolEnd = this._pool.length;
-    }
-    var start = this._poolStart;
+    var chunkBytes = 0,
+        bytesRead = 0,
+        start = this._buffer.offset;
 
     do {
-      chunkBytes = this._pusher(this._pool,
-                                this._poolStart,
-                                this._poolEnd - this._poolStart);
+      chunkBytes = this._buffer.use(this, this._pusher);
+      if (chunkBytes > 0) bytesRead += chunkBytes;
 
       if (this.pair.ssl && this.pair.ssl.error) {
         this.pair.error();
@@ -359,13 +508,12 @@ CryptoStream.prototype._push = function() {
 
       this.pair.maybeInitFinished();
 
-      if (chunkBytes >= 0) {
-        this._poolStart += chunkBytes;
-      }
+    } while (chunkBytes > 0 && !this._buffer.isFull);
 
-    } while (chunkBytes > 0 && this._poolStart < this._poolEnd);
+    var pool = this._buffer.pool;
 
-    var bytesRead = this._poolStart - start;
+    // Create new buffer if previous was filled up
+    if (this._buffer.isFull) this._buffer.create();
 
     assert(bytesRead >= 0);
 
@@ -377,7 +525,7 @@ CryptoStream.prototype._push = function() {
       return;
     }
 
-    var chunk = this._pool.slice(start, this._poolStart);
+    var chunk = pool.slice(start, start + bytesRead);
 
     if (this === this.pair.cleartext) {
       debug('cleartext emit "data" with ' + bytesRead + ' bytes');
@@ -393,7 +541,7 @@ CryptoStream.prototype._push = function() {
     }
 
     // Optimization: emit the original buffer with end points
-    if (this.ondata) this.ondata(this._pool, start, this._poolStart);
+    if (this.ondata) this.ondata(pool, start, start + bytesRead);
   }
 };
 
@@ -935,7 +1083,7 @@ function Server(/* [options], listener */) {
     passphrase: self.passphrase,
     cert: self.cert,
     ca: self.ca,
-    ciphers: self.ciphers || 'RC4-SHA:AES128-SHA:AES256-SHA',
+    ciphers: self.ciphers || DEFAULT_CIPHERS,
     secureProtocol: self.secureProtocol,
     secureOptions: self.secureOptions,
     crl: self.crl,
@@ -969,7 +1117,7 @@ function Server(/* [options], listener */) {
       } else {
         var verifyError = pair.ssl.verifyError();
         if (verifyError) {
-          pair.cleartext.authorizationError = verifyError;
+          pair.cleartext.authorizationError = verifyError.message;
 
           if (self.rejectUnauthorized) {
             socket.destroy();
@@ -1084,45 +1232,36 @@ Server.prototype.SNICallback = function(servername) {
 //  });
 //
 //
+function normalizeConnectArgs(listArgs) {
+  var args = net._normalizeConnectArgs(listArgs);
+  var options = args[0];
+  var cb = args[1];
+
+  if (typeof listArgs[1] === 'object') {
+    options = util._extend(options, listArgs[1]);
+  } else if (typeof listArgs[2] === 'object') {
+    options = util._extend(options, listArgs[2]);
+  }
+
+  return (cb) ? [options, cb] : [options];
+}
+
 exports.connect = function(/* [port, host], options, cb */) {
-  var options, port, host, cb;
-
-  if (typeof arguments[0] === 'object') {
-    options = arguments[0];
-  } else if (typeof arguments[1] === 'object') {
-    options = arguments[1];
-    port = arguments[0];
-  } else if (typeof arguments[2] === 'object') {
-    options = arguments[2];
-    port = arguments[0];
-    host = arguments[1];
-  } else {
-    // This is what happens when user passes no `options` argument, we can't
-    // throw `TypeError` here because it would be incompatible with old API
-    if (typeof arguments[0] === 'number') {
-      port = arguments[0];
-    }
-    if (typeof arguments[1] === 'string') {
-      host = arguments[1];
-    }
-  }
-
-  options = util._extend({ port: port, host: host }, options || {});
-
-  if (typeof arguments[arguments.length - 1] === 'function') {
-    cb = arguments[arguments.length - 1];
-  }
+  var args = normalizeConnectArgs(arguments);
+  var options = args[0];
+  var cb = args[1];
 
   var socket = options.socket ? options.socket : new net.Stream();
 
   var sslcontext = crypto.createCredentials(options);
 
   convertNPNProtocols(options.NPNProtocols, this);
-  var pair = new SecurePair(sslcontext, false, true,
+  var hostname = options.servername || options.host || 'localhost',
+      pair = new SecurePair(sslcontext, false, true,
                             options.rejectUnauthorized === true ? true : false,
                             {
                               NPNProtocols: this.NPNProtocols,
-                              servername: options.servername || options.host
+                              servername: hostname
                             });
 
   if (options.session) {
@@ -1131,15 +1270,16 @@ exports.connect = function(/* [port, host], options, cb */) {
 
   var cleartext = pipe(pair, socket);
   if (cb) {
-    cleartext.on('secureConnect', cb);
+    cleartext.once('secureConnect', cb);
   }
 
   if (!options.socket) {
-    socket.connect({
+    var connect_opt = (options.path && !options.port) ? {path: options.path} : {
       port: options.port,
       host: options.host,
       localAddress: options.localAddress
-    });
+    };
+    socket.connect(connect_opt);
   }
 
   pair.on('secure', function() {
@@ -1147,9 +1287,19 @@ exports.connect = function(/* [port, host], options, cb */) {
 
     cleartext.npnProtocol = pair.npnProtocol;
 
+    // Verify that server's identity matches it's certificate's names
+    if (!verifyError) {
+      var validCert = checkServerIdentity(hostname,
+                                          pair.cleartext.getPeerCertificate());
+      if (!validCert) {
+        verifyError = new Error('Hostname/IP doesn\'t match certificate\'s ' +
+                                'altnames');
+      }
+    }
+
     if (verifyError) {
       cleartext.authorized = false;
-      cleartext.authorizationError = verifyError;
+      cleartext.authorizationError = verifyError.message;
 
       if (pair._rejectUnauthorized) {
         cleartext.emit('error', verifyError);
