@@ -685,6 +685,24 @@ void AfterGetAddrInfo(uv_getaddrinfo_t* req, int status, struct addrinfo* res) {
 }
 
 
+static Handle<Value> IsIP(const Arguments& args) {
+  HandleScope scope;
+
+  String::AsciiValue ip(args[0]);
+  char address_buffer[sizeof(struct in6_addr)];
+
+  if (uv_inet_pton(AF_INET, *ip, &address_buffer) == 1) {
+    return scope.Close(v8::Integer::New(4));
+  }
+
+  if (uv_inet_pton(AF_INET6, *ip, &address_buffer) == 1) {
+    return scope.Close(v8::Integer::New(6));
+  }
+
+  return scope.Close(v8::Integer::New(0));
+}
+
+
 static Handle<Value> GetAddrInfo(const Arguments& args) {
   HandleScope scope;
 
@@ -750,6 +768,7 @@ static void Initialize(Handle<Object> target) {
   NODE_SET_METHOD(target, "getHostByName", QueryWithFamily<GetHostByNameWrap>);
 
   NODE_SET_METHOD(target, "getaddrinfo", GetAddrInfo);
+  NODE_SET_METHOD(target, "isIP", IsIP);
 
   target->Set(String::NewSymbol("AF_INET"), Integer::New(AF_INET));
   target->Set(String::NewSymbol("AF_INET6"), Integer::New(AF_INET6));
